//============================================================================================
// frhed - free hex editor

#include <windows.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <direct.h>
#include <string.h>
#include <stdio.h>
#include <commctrl.h>
#include <objbase.h>
#include <shlobj.h>

HINSTANCE hMainInstance;

#include "resource.h"
#include "simparr.cpp"
#include "diagbox.cpp"
#include "hexwnd.cpp"

LRESULT CALLBACK MainWndProc (HWND, UINT, WPARAM, LPARAM);

//============================================================================================
// The main window object.
HexEditorWindow hexwnd;

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, char* szCmdLine, int iCmdShow)
{
	hMainInstance = hInstance;

	HACCEL hAccel;

	static char szAppName[] = "frhed wndclass";
	HWND hWnd;
	MSG msg;
	WNDCLASSEX wndclass;

	wndclass.cbSize = sizeof (wndclass);
	wndclass.style = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc = MainWndProc;
	wndclass.cbClsExtra = 0;
	wndclass.cbWndExtra = 0;
	wndclass.hInstance = hInstance;
	wndclass.hIcon = LoadIcon (hInstance, MAKEINTRESOURCE (IDI_ICON1));
	wndclass.hCursor = LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground = NULL;
	wndclass.lpszMenuName = MAKEINTRESOURCE (IDR_MAINMENU);
	wndclass.lpszClassName = szAppName;
	wndclass.hIconSm = LoadImage (hInstance, MAKEINTRESOURCE (IDI_ICON1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);

	RegisterClassEx (&wndclass);

	hWnd = CreateWindow (szAppName,
		"frhed window",
		WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
		hexwnd.iWindowX,
		hexwnd.iWindowY,
		hexwnd.iWindowWidth,
		hexwnd.iWindowHeight,
		NULL,
		NULL,
		hInstance,
		NULL);

	ShowWindow( hWnd, hexwnd.iWindowShowCmd );
	UpdateWindow (hWnd);

	if( szCmdLine != NULL && strlen( szCmdLine ) != 0 )
	{
		// Command line not empty: open a file on startup.
		char lpszPath[MAX_PATH];
		HRESULT hres = ResolveIt( hWnd, szCmdLine, lpszPath );
		if( SUCCEEDED( hres ) )
		{
			// Trying to open a link file: decision by user required.
			int ret = MessageBox( hWnd,
				"You are trying to open a link file.\n"
				"Click on Yes if you want to open the file linked to,\n"
				"or click on No if you want to open the link file itself.\n"
				"Choose Cancel if you want to abort opening.",
				"frhed", MB_YESNOCANCEL | MB_ICONQUESTION );
			switch( ret )
			{
			case IDYES:
				hexwnd.load_file( lpszPath );
				break;
			case IDNO:
				hexwnd.load_file( szCmdLine );
				break;
			case IDCANCEL:
				break;
			}
		}
		else
		{
			hexwnd.load_file( szCmdLine );
		}
	}

	hAccel = LoadAccelerators (hInstance, MAKEINTRESOURCE (IDR_ACCELERATOR1));

	while (GetMessage (&msg, NULL, 0, 0))
	{
		if (!TranslateAccelerator (hWnd, hAccel, &msg))
		{
			TranslateMessage (&msg);
			DispatchMessage (&msg);
		}
	}
	return msg.wParam;
}

//--------------------------------------------------------------------------------------------
// The main window procedure.
LRESULT CALLBACK MainWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	switch (iMsg)
	{
	case WM_CREATE:
		hexwnd.at_window_create (hwnd, hMainInstance);
		return 0;

	case WM_SIZE:
		hexwnd.resize_window (LOWORD (lParam), HIWORD (lParam));
		return 0;

	case WM_SETFOCUS:
		hexwnd.set_focus ();
		return 0;

	case WM_KILLFOCUS:
		hexwnd.kill_focus ();
		return 0;

	case WM_LBUTTONDOWN:
		hexwnd.lbuttondown (LOWORD(lParam), HIWORD(lParam));
		return 0;

	case WM_LBUTTONUP:
		hexwnd.lbuttonup (LOWORD(lParam), HIWORD(lParam));
		return 0;

	case WM_MOUSEMOVE:
		hexwnd.mousemove ((int) (short) LOWORD(lParam), (int) (short) HIWORD(lParam));
		return 0;
	
	case WM_KEYDOWN:
		hexwnd.keydown (wParam);
		return 0;

	case WM_CHAR:
		hexwnd.character (wParam);
		return 0;

	case WM_VSCROLL:
		hexwnd.vscroll (LOWORD (wParam), HIWORD (wParam));
		return 0;

	case WM_HSCROLL:
		hexwnd.hscroll (LOWORD (wParam), HIWORD (wParam));
		return 0;

	case WM_PAINT:
		hexwnd.paint ();
		return 0;

	case WM_COMMAND:
		hexwnd.command (LOWORD (wParam));
		return 0;

	case WM_INITMENUPOPUP:
		hexwnd.initmenupopup (wParam, lParam);
		return 0;

	case WM_TIMER:
		hexwnd.timer (wParam, lParam);
		return 0;

	case WM_DROPFILES:
		hexwnd.dropfiles( (HANDLE) wParam ); // handle of internal drop structure.
		return 0;

	case WM_CLOSE:
		hexwnd.close ();
		return 0;

	case WM_DESTROY:
		hexwnd.destroy_window ();
		PostQuitMessage (0);
		return 0;
	}
	return DefWindowProc (hwnd, iMsg, wParam, lParam);
}

//============================================================================================
